#include <algorithm>
#include <iostream>
#include <math.h>
#include "Nbody.h"


//-----------------------------------------------------------------------------
Nbody::Nbody(int _Nstar) {
  Nstar = _Nstar;
  stardata = new Star[Nstar];
}


//-----------------------------------------------------------------------------
Nbody::~Nbody() {
  delete[] stardata;
}


//-----------------------------------------------------------------------------
void Nbody::CalculateAccelerations(void) {
  int i,j,k;
  float dr[ndim],rsqd,rmag;

  for (i=0; i<Nstar; i++) {
    stardata[i].potential = 0.0f;
    for (k=0; k<ndim; k++) stardata[i].a[k] = 0.0f;

    for (j=0; j<Nstar; j++) {
      if (i == j) continue;

      for (k=0; k<ndim; k++) dr[k] = stardata[i].r[k] - stardata[j].r[k];
      rsqd = dr[0]*dr[0] + dr[1]*dr[1] + dr[2]*dr[2];
      rmag = sqrtf(rsqd);

      for (k=0; k<ndim; k++) 
        stardata[i].a[k] -= Gconst*stardata[j].m*dr[k]/rmag/(rsqd + epsilon*epsilon);
      stardata[i].potential += Gconst*stardata[j].m/sqrtf(rsqd + epsilon*epsilon);

    }

  }

  return;
}



//-----------------------------------------------------------------------------
NbodyEuler::NbodyEuler(int _Nstar) : Nbody(_Nstar) 
{
}


//-----------------------------------------------------------------------------
void NbodyEuler::IntegrationStep(float dt)
{
  int i,k;

  CalculateAccelerations();

  for (i=0; i<Nstar; i++) {
    for (k=0; k<ndim; k++) stardata[i].r[k] += stardata[i].v[k]*dt;
    for (k=0; k<ndim; k++) stardata[i].v[k] += stardata[i].a[k]*dt;
  }

  return;
}



//-----------------------------------------------------------------------------
NbodyLeapfrog::NbodyLeapfrog(int _Nstar) : Nbody(_Nstar) 
{
}


//-----------------------------------------------------------------------------
void NbodyLeapfrog::IntegrationStep(float dt)
{
  int i,k;

  for (i=0; i<Nstar; i++) {
    for (k=0; k<ndim; k++) stardata[i].a0[k] = stardata[i].a[k]*dt;
    for (k=0; k<ndim; k++) 
      stardata[i].r[k] += stardata[i].v[k]*dt + 0.5f*stardata[i].a[k]*dt*dt;
  }

  CalculateAccelerations();

  for (i=0; i<Nstar; i++) {
    for (k=0; k<ndim; k++) 
      stardata[i].v[k] += 0.5f*(stardata[i].a0[k] + stardata[i].a[k])*dt;
  }

  return;
}



