#include <cstdlib>
#include <fstream>
#include <iostream>
#include "NbodySimulation.h"


//-----------------------------------------------------------------------------
NbodySimulation::NbodySimulation()
{
  t = 0.0f;
}


//-----------------------------------------------------------------------------
NbodySimulation::~NbodySimulation()
{
}


//-----------------------------------------------------------------------------
void NbodySimulation::ReadParametersFile(std::string paramsfile)
{
  std::ifstream infile;

  infile.open(paramsfile.c_str());
  infile >> algorithm;
  infile >> ic;
  infile >> Nstar;
  infile >> epsilon;
  infile >> vmax;
  infile >> dt;
  infile >> simtime;
  infile.close();

  return;
}


//-----------------------------------------------------------------------------
void NbodySimulation::Setup()
{
  int i,k;

  // Create N-body algorithm object based on choice made in parameters file
  if (algorithm == "euler") {
    nbody = new NbodyEuler(Nstar);
  }
  else {
    std::cout << "Unrecognised algorithm" << std::endl;
    exit(0);
  }

  // Prepare simple 'random' initial conditions
  if (ic == "random") {
    for (i=0; i<Nstar; i++) {
      for (k=0; k<ndim; k++) nbody->stardata[i].r[k] =
        2.0f*(float) (rand()%RAND_MAX)/(float) RAND_MAX - 1.0f;
      for (k=0; k<ndim; k++) nbody->stardata[i].v[k] =
        vmax*2.0f*(float) (rand()%RAND_MAX)/(float) RAND_MAX - 1.0f;
      nbody->stardata[i].m = 1.0f / (float) Nstar;
      nbody->stardata[i].epsilon = epsilon;
    }
  }
  else if (ic == "binary") {
    for (i=0; i<Nstar; i++) {
      for (k=0; k<ndim; k++) nbody->stardata[i].r[k] = 0.0f;
      for (k=0; k<ndim; k++) nbody->stardata[i].v[k] = 0.0f;
      nbody->stardata[i].m = 1.0f / (float) Nstar;
      nbody->stardata[i].epsilon = epsilon;
    }
    nbody->stardata[0].r[0] = -0.5f;
    nbody->stardata[0].v[1] = -0.5f;
    nbody->stardata[1].r[0] = 0.5f;
    nbody->stardata[1].v[1] = 0.5f;
  }

  nbody->CalculateAccelerations();

  return;
}


//-----------------------------------------------------------------------------
void NbodySimulation::Run()
{
  return;
}


//-----------------------------------------------------------------------------
void NbodySimulation::Finish()
{
  delete nbody;
  return;
}
